# Copyright (c) 2008,2009 Citrix Systems, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published
# by the Free Software Foundation; version 2.1 only. with the special
# exception on linking described in file LICENSE.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
import sys
import syslog
import os

from xml.dom.minidom import getDOMImplementation
from xml.dom.minidom import parse as parseXML

the_root_prefix = ""
def root_prefix():
    """Returns a string to prefix to all file name references, which
    is useful for testing."""
    return the_root_prefix
def set_root_prefix(prefix):
    global the_root_prefix
    the_root_prefix = prefix

log_destination = "syslog"
def get_log_destination():
    """Returns the current log destination.
    'syslog' means "log to syslog".
    'stderr' means "log to stderr"."""
    return log_destination
def set_log_destination(dest):
    global log_destination
    log_destination = dest

#
# Logging.
#

def log(s):
    if get_log_destination() == 'syslog':
        syslog.syslog(s)
    else:
        sys.stderr.write(s + '\n')
        sys.stderr.flush()

#
# Exceptions.
#

class Error(Exception):
    def __init__(self, msg):
        Exception.__init__(self)
        self.msg = msg

#
# Run external utilities
#

def run_command(command):
    log("Running command: " + ' '.join(command))
    rc = os.spawnl(os.P_WAIT, root_prefix() + command[0], *command)
    if rc != 0:
        log("Command failed %d: " % rc + ' '.join(command))
        return False
    return True

#
# Configuration File Handling.
#

class ConfigurationFile(object):
    """Write a file, tracking old and new versions.

    Supports writing a new version of a file and applying and
    reverting those changes.
    """

    __STATE = {"OPEN":"OPEN",
               "NOT-APPLIED":"NOT-APPLIED", "APPLIED":"APPLIED",
               "REVERTED":"REVERTED", "COMMITTED": "COMMITTED"}

    def __init__(self, path):
        dirname,basename = os.path.split(path)

        self.__state = self.__STATE['OPEN']
        self.__children = []

        self.__path    = os.path.join(dirname, basename)
        self.__oldpath = os.path.join(dirname, "." + basename + ".xapi-old")
        self.__newpath = os.path.join(dirname, "." + basename + ".xapi-new")

        self.__f = open(self.__newpath, "w")

    def attach_child(self, child):
        self.__children.append(child)

    def path(self):
        return self.__path

    def readlines(self):
        try:
            return open(self.path()).readlines()
        except:
            return ""

    def write(self, args):
        if self.__state != self.__STATE['OPEN']:
            raise Error("Attempt to write to file in state %s" % self.__state)
        self.__f.write(args)

    def close(self):
        if self.__state != self.__STATE['OPEN']:
            raise Error("Attempt to close file in state %s" % self.__state)

        self.__f.close()
        self.__state = self.__STATE['NOT-APPLIED']

    def changed(self):
        if self.__state != self.__STATE['NOT-APPLIED']:
            raise Error("Attempt to compare file in state %s" % self.__state)

        return True

    def apply(self):
        if self.__state != self.__STATE['NOT-APPLIED']:
            raise Error("Attempt to apply configuration from state %s" % self.__state)

        for child in self.__children:
            child.apply()

        log("Applying changes to %s configuration" % self.__path)

        # Remove previous backup.
        if os.access(self.__oldpath, os.F_OK):
            os.unlink(self.__oldpath)

        # Save current configuration.
        if os.access(self.__path, os.F_OK):
            os.link(self.__path, self.__oldpath)
            os.unlink(self.__path)

        # Apply new configuration.
        assert(os.path.exists(self.__newpath))
        os.link(self.__newpath, self.__path)

        # Remove temporary file.
        os.unlink(self.__newpath)

        self.__state = self.__STATE['APPLIED']

    def revert(self):
        if self.__state != self.__STATE['APPLIED']:
            raise Error("Attempt to revert configuration from state %s" % self.__state)

        for child in self.__children:
            child.revert()

        log("Reverting changes to %s configuration" % self.__path)

        # Remove existing new configuration
        if os.access(self.__newpath, os.F_OK):
            os.unlink(self.__newpath)

        # Revert new configuration.
        if os.access(self.__path, os.F_OK):
            os.link(self.__path, self.__newpath)
            os.unlink(self.__path)

        # Revert to old configuration.
        if os.access(self.__oldpath, os.F_OK):
            os.link(self.__oldpath, self.__path)
            os.unlink(self.__oldpath)

        # Leave .*.xapi-new as an aid to debugging.

        self.__state = self.__STATE['REVERTED']

    def commit(self):
        if self.__state != self.__STATE['APPLIED']:
            raise Error("Attempt to commit configuration from state %s" % self.__state)

        for child in self.__children:
            child.commit()

        log("Committing changes to %s configuration" % self.__path)

        if os.access(self.__oldpath, os.F_OK):
            os.unlink(self.__oldpath)
        if os.access(self.__newpath, os.F_OK):
            os.unlink(self.__newpath)

        self.__state = self.__STATE['COMMITTED']

#
# Helper functions for encoding/decoding database attributes to/from XML.
#

def _str_to_xml(xml, parent, tag, val):
    e = xml.createElement(tag)
    parent.appendChild(e)
    v = xml.createTextNode(val)
    e.appendChild(v)
def _str_from_xml(n):
    def getText(nodelist):
        rc = ""
        for node in nodelist:
            if node.nodeType == node.TEXT_NODE:
                rc = rc + node.data
        return rc
    return getText(n.childNodes).strip()

def _bool_to_xml(xml, parent, tag, val):
    if val:
        _str_to_xml(xml, parent, tag, "True")
    else:
        _str_to_xml(xml, parent, tag, "False")
def _bool_from_xml(n):
    s = _str_from_xml(n)
    if s == "True":
        return True
    elif s == "False":
        return False
    else:
        raise Error("Unknown boolean value %s" % s)

def _strlist_to_xml(xml, parent, ltag, itag, val):
    e = xml.createElement(ltag)
    parent.appendChild(e)
    for v in val:
        c = xml.createElement(itag)
        e.appendChild(c)
        cv = xml.createTextNode(v)
        c.appendChild(cv)
def _strlist_from_xml(n, ltag, itag):
    ret = []
    for n in n.childNodes:
        if n.nodeName == itag:
            ret.append(_str_from_xml(n))
    return ret

def _map_to_xml(xml, parent, tag, val, attrs):
    e = xml.createElement(tag)
    parent.appendChild(e)
    for n,v in val.items():
        if n in attrs:
            _str_to_xml(xml, e, n, v)
        else:
            log("Unknown other-config attribute: %s" % n)

def _map_from_xml(n, attrs):
    ret = {}
    for n in n.childNodes:
        if n.nodeName in attrs:
            ret[n.nodeName] = _str_from_xml(n)
    return ret

def _otherconfig_to_xml(xml, parent, val, attrs):
    return _map_to_xml(xml, parent, "other_config", val, attrs)
def _otherconfig_from_xml(n, attrs):
    return _map_from_xml(n, attrs)

#
# Definitions of the database objects (and their attributes) used by interface-reconfigure.
#
# Each object is defined by a dictionary mapping an attribute name in
# the xapi database to a tuple containing two items:
#  - a function which takes this attribute and encodes it as XML.
#  - a function which takes XML and decocdes it into a value.
#
# other-config attributes are specified as a simple array of strings

_PIF_XML_TAG = "pif"
_VLAN_XML_TAG = "vlan"
_TUNNEL_XML_TAG = "tunnel"
_BOND_XML_TAG = "bond"
_NETWORK_XML_TAG = "network"
_POOL_XML_TAG = "pool"

_ETHTOOL_OTHERCONFIG_ATTRS = ['ethtool-%s' % x for x in ['autoneg', 'speed', 'duplex', 'rx', 'tx', 'sg', 'tso', 'ufo', 'gso', 'gro', 'lro'] ]

_PIF_OTHERCONFIG_ATTRS = [ 'domain', 'peerdns', 'defaultroute', 'mtu', 'static-routes' ] + \
                        [ 'bond-%s' % x for x in ['mode', 'miimon', 'downdelay', 'updelay', 'use_carrier', 'hashing-algorithm'] ] + \
                        [ 'vlan-bug-workaround' ] + \
                        _ETHTOOL_OTHERCONFIG_ATTRS

_PIF_ATTRS = { 'uuid': (_str_to_xml,_str_from_xml),
               'management': (_bool_to_xml,_bool_from_xml),
               'network': (_str_to_xml,_str_from_xml),
               'device': (_str_to_xml,_str_from_xml),
               'bond_master_of': (lambda x, p, t, v: _strlist_to_xml(x, p, 'bond_master_of', 'slave', v),
                                  lambda n: _strlist_from_xml(n, 'bond_master_of', 'slave')),
               'bond_slave_of': (_str_to_xml,_str_from_xml),
               'VLAN': (_str_to_xml,_str_from_xml),
               'VLAN_master_of': (_str_to_xml,_str_from_xml),
               'VLAN_slave_of': (lambda x, p, t, v: _strlist_to_xml(x, p, 'VLAN_slave_of', 'master', v),
                                 lambda n: _strlist_from_xml(n, 'VLAN_slave_Of', 'master')),
               'tunnel_access_PIF_of': (lambda x, p, t, v: _strlist_to_xml(x, p, 'tunnel_access_PIF_of', 'pif', v),
                                        lambda n: _strlist_from_xml(n, 'tunnel_access_PIF_of', 'pif')),
               'tunnel_transport_PIF_of':  (lambda x, p, t, v: _strlist_to_xml(x, p, 'tunnel_transport_PIF_of', 'pif', v),
                                            lambda n: _strlist_from_xml(n, 'tunnel_transport_PIF_of', 'pif')),
               'ip_configuration_mode': (_str_to_xml,_str_from_xml),
               'IP': (_str_to_xml,_str_from_xml),
               'netmask': (_str_to_xml,_str_from_xml),
               'gateway': (_str_to_xml,_str_from_xml),
               'DNS': (_str_to_xml,_str_from_xml),
               'MAC': (_str_to_xml,_str_from_xml),
               'other_config': (lambda x, p, t, v: _otherconfig_to_xml(x, p, v, _PIF_OTHERCONFIG_ATTRS),
                                lambda n: _otherconfig_from_xml(n, _PIF_OTHERCONFIG_ATTRS)),

               # Special case: We write the current value
               # PIF.currently-attached to the cache but since it will
               # not be valid when we come to use the cache later
               # (i.e. after a reboot) we always read it as False.
               'currently_attached': (_bool_to_xml, lambda n: False),
             }

_VLAN_ATTRS = { 'uuid': (_str_to_xml,_str_from_xml),
                'tagged_PIF': (_str_to_xml,_str_from_xml),
                'untagged_PIF': (_str_to_xml,_str_from_xml),
              }

_TUNNEL_ATTRS = { 'uuid': (_str_to_xml,_str_from_xml),
                  'access_PIF': (_str_to_xml,_str_from_xml),
                  'transport_PIF': (_str_to_xml,_str_from_xml),
                }
_BOND_ATTRS = { 'uuid': (_str_to_xml,_str_from_xml),
               'master': (_str_to_xml,_str_from_xml),
               'slaves': (lambda x, p, t, v: _strlist_to_xml(x, p, 'slaves', 'slave', v),
                          lambda n: _strlist_from_xml(n, 'slaves', 'slave')),
              }

_NETWORK_OTHERCONFIG_ATTRS = [ 'mtu',
                               'static-routes',
                               'vswitch-controller-fail-mode',
                               'vswitch-disable-in-band' ] \
                               + _ETHTOOL_OTHERCONFIG_ATTRS

_NETWORK_ATTRS = { 'uuid': (_str_to_xml,_str_from_xml),
                   'bridge': (_str_to_xml,_str_from_xml),
                   'MTU': (_str_to_xml,_str_from_xml),
                   'PIFs': (lambda x, p, t, v: _strlist_to_xml(x, p, 'PIFs', 'PIF', v),
                            lambda n: _strlist_from_xml(n, 'PIFs', 'PIF')),
                   'other_config': (lambda x, p, t, v: _otherconfig_to_xml(x, p, v, _NETWORK_OTHERCONFIG_ATTRS),
                                    lambda n: _otherconfig_from_xml(n, _NETWORK_OTHERCONFIG_ATTRS)),
                 }

_POOL_OTHERCONFIG_ATTRS = ['vswitch-controller-fail-mode']

_POOL_ATTRS = { 'other_config': (lambda x, p, t, v: _otherconfig_to_xml(x, p, v, _POOL_OTHERCONFIG_ATTRS),
                                 lambda n: _otherconfig_from_xml(n, _POOL_OTHERCONFIG_ATTRS)),
              }

#
# Database Cache object
#

_db = None

def db():
    assert(_db is not None)
    return _db

def db_init_from_cache(cache):
    global _db
    assert(_db is None)
    _db = DatabaseCache(cache_file=cache)
    
def db_init_from_xenapi(session):
    global _db 
    assert(_db is None)
    _db  = DatabaseCache(session_ref=session)
    
class DatabaseCache(object):
    def __read_xensource_inventory(self):
        filename = root_prefix() + "/etc/xensource-inventory"
        f = open(filename, "r")
        lines = [x.strip("\n") for x in f.readlines()]
        f.close()

        defs = [ (l[:l.find("=")], l[(l.find("=") + 1):]) for l in lines ]
        defs = [ (a, b.strip("'")) for (a,b) in defs ]

        return dict(defs)

    def __pif_on_host(self,pif):
        return pif in self.__pifs

    def __get_pif_records_from_xapi(self, session, host):
        self.__pifs = {}
        for (p,rec) in session.xenapi.PIF.get_all_records().items():
            if rec['host'] != host:
                continue
            self.__pifs[p] = {}
            for f in _PIF_ATTRS:
                self.__pifs[p][f] = rec[f]
            self.__pifs[p]['other_config'] = {}
            for f in _PIF_OTHERCONFIG_ATTRS:
                if f not in rec['other_config']: continue
                self.__pifs[p]['other_config'][f] = rec['other_config'][f]

    def __get_vlan_records_from_xapi(self, session):
        self.__vlans = {}
        for (v,rec) in session.xenapi.VLAN.get_all_records().items():
            if not self.__pif_on_host(rec['untagged_PIF']):
                continue
            self.__vlans[v] = {}
            for f in _VLAN_ATTRS:
                self.__vlans[v][f] = rec[f]

    def __get_tunnel_records_from_xapi(self, session):
        self.__tunnels = {}
        for t in session.xenapi.tunnel.get_all():
            rec = session.xenapi.tunnel.get_record(t)
            if not self.__pif_on_host(rec['transport_PIF']):
                continue
            self.__tunnels[t] = {}
            for f in _TUNNEL_ATTRS:
                self.__tunnels[t][f] = rec[f]

    def __get_bond_records_from_xapi(self, session):
        self.__bonds = {}
        for (b,rec) in session.xenapi.Bond.get_all_records().items():
            if not self.__pif_on_host(rec['master']):
                continue
            self.__bonds[b] = {}
            for f in _BOND_ATTRS:
                self.__bonds[b][f] = rec[f]

    def __get_network_records_from_xapi(self, session):
        self.__networks = {}
        for (n,rec) in session.xenapi.network.get_all_records().items():
            self.__networks[n] = {}
            for f in _NETWORK_ATTRS:
                if f == "PIFs":
                    # drop PIFs on other hosts
                    self.__networks[n][f] = [p for p in rec[f] if self.__pif_on_host(p)]
                elif f == "MTU" and f not in rec:
                    # XenServer 5.5 network records did not have an
                    # MTU field, so allow this to be missing.
                    pass
                else:
                    self.__networks[n][f] = rec[f]
            self.__networks[n]['other_config'] = {}
            for f in _NETWORK_OTHERCONFIG_ATTRS:
                if f not in rec['other_config']: continue
                self.__networks[n]['other_config'][f] = rec['other_config'][f]

    def __get_pool_records_from_xapi(self, session):
        self.__pools = {}
        for p in session.xenapi.pool.get_all():
            rec = session.xenapi.pool.get_record(p)

            self.__pools[p] = {}

            for f in _POOL_ATTRS:
                self.__pools[p][f] = rec[f]

            for f in _POOL_OTHERCONFIG_ATTRS:
                if f in rec['other_config']:
                    self.__pools[p]['other_config'][f] = rec['other_config'][f]

    def __to_xml(self, xml, parent, key, ref, rec, attrs):
        """Encode a database object as XML"""
        e = xml.createElement(key)
        parent.appendChild(e)
        if ref:
            e.setAttribute('ref', ref)

        for n,v in rec.items():
            if n in attrs:
                h,_ = attrs[n]
                h(xml, e, n, v)
            else:
                raise Error("Unknown attribute %s" % n)
    def __from_xml(self, e, attrs):
        """Decode a database object from XML"""
        ref = e.attributes['ref'].value
        rec = {}
        for n in e.childNodes:
            if n.nodeName in attrs:
                _,h = attrs[n.nodeName]
                rec[n.nodeName] = h(n)
        return (ref,rec)

    def __init__(self, session_ref=None, cache_file=None):
        if session_ref and cache_file:
            raise Error("can't specify session reference and cache file")
        if cache_file == None:
            import XenAPI
            session = XenAPI.xapi_local()

            if not session_ref:
                log("No session ref given on command line, logging in.")
                session.xenapi.login_with_password("root", "")
            else:
                session._session = session_ref

            try:

                inventory = self.__read_xensource_inventory()
                assert('INSTALLATION_UUID' in inventory)
                log("host uuid is %s" % inventory['INSTALLATION_UUID'])

                host = session.xenapi.host.get_by_uuid(inventory['INSTALLATION_UUID'])

                self.__get_pif_records_from_xapi(session, host)
                self.__get_pool_records_from_xapi(session)
                self.__get_tunnel_records_from_xapi(session)
                self.__get_vlan_records_from_xapi(session)
                self.__get_bond_records_from_xapi(session)
                self.__get_network_records_from_xapi(session)
            finally:
                if not session_ref:
                    session.xenapi.session.logout()
        else:
            log("Loading xapi database cache from %s" % cache_file)

            xml = parseXML(root_prefix() + cache_file)

            self.__pifs = {}
            self.__bonds = {}
            self.__vlans = {}
            self.__pools = {}
            self.__tunnels = {}
            self.__networks = {}

            assert(len(xml.childNodes) == 1)
            toplevel = xml.childNodes[0]

            assert(toplevel.nodeName == "xenserver-network-configuration")

            for n in toplevel.childNodes:
                if n.nodeName == "#text":
                    pass
                elif n.nodeName == _PIF_XML_TAG:
                    (ref,rec) = self.__from_xml(n, _PIF_ATTRS)
                    self.__pifs[ref] = rec
                elif n.nodeName == _BOND_XML_TAG:
                    (ref,rec) = self.__from_xml(n, _BOND_ATTRS)
                    self.__bonds[ref] = rec
                elif n.nodeName == _VLAN_XML_TAG:
                    (ref,rec) = self.__from_xml(n, _VLAN_ATTRS)
                    self.__vlans[ref] = rec
                elif n.nodeName == _TUNNEL_XML_TAG:
                    (ref,rec) = self.__from_xml(n, _TUNNEL_ATTRS)
                    self.__vlans[ref] = rec
                elif n.nodeName == _NETWORK_XML_TAG:
                    (ref,rec) = self.__from_xml(n, _NETWORK_ATTRS)
                    self.__networks[ref] = rec
                elif n.nodeName == _POOL_XML_TAG:
                    (ref,rec) = self.__from_xml(n, _POOL_ATTRS)
                    self.__pools[ref] = rec
                else:
                    raise Error("Unknown XML element %s" % n.nodeName)

    def save(self, cache_file):

        xml = getDOMImplementation().createDocument(
            None, "xenserver-network-configuration", None)
        for (ref,rec) in self.__pifs.items():
            self.__to_xml(xml, xml.documentElement, _PIF_XML_TAG, ref, rec, _PIF_ATTRS)
        for (ref,rec) in self.__bonds.items():
            self.__to_xml(xml, xml.documentElement, _BOND_XML_TAG, ref, rec, _BOND_ATTRS)
        for (ref,rec) in self.__vlans.items():
            self.__to_xml(xml, xml.documentElement, _VLAN_XML_TAG, ref, rec, _VLAN_ATTRS)
        for (ref,rec) in self.__tunnels.items():
            self.__to_xml(xml, xml.documentElement, _TUNNEL_XML_TAG, ref, rec, _TUNNEL_ATTRS)
        for (ref,rec) in self.__networks.items():
            self.__to_xml(xml, xml.documentElement, _NETWORK_XML_TAG, ref, rec,
                          _NETWORK_ATTRS)
        for (ref,rec) in self.__pools.items():
            self.__to_xml(xml, xml.documentElement, _POOL_XML_TAG, ref, rec, _POOL_ATTRS)

        temp_file = cache_file + ".%d" % os.getpid()
        f = open(temp_file, 'w')
        f.write(xml.toprettyxml())
        f.close()
        os.rename(temp_file, cache_file)

    def get_pif_by_uuid(self, uuid):
        pifs = map(lambda ref_rec: ref_rec[0],
                  filter(lambda ref_rec: uuid == ref_rec[1]['uuid'],
                         self.__pifs.items()))
        if len(pifs) == 0:
            raise Error("Unknown PIF \"%s\"" % uuid)
        elif len(pifs) > 1:
            raise Error("Non-unique PIF \"%s\"" % uuid)

        return pifs[0]

    def get_pifs_by_device(self, device):
        return list(map(lambda ref_rec: ref_rec[0],
                   list(filter(lambda ref_rec: ref_rec[1]['device'] == device,
                          self.__pifs.items()))))

    def get_networks_with_bridge(self, bridge):
        return list(map(lambda ref_rec: ref_rec[0],
                  list(filter(lambda ref_rec: ref_rec[1]['bridge'] == bridge,
                         self.__networks.items()))))

    def get_network_by_bridge(self, bridge):
        #Assumes one network has bridge.
        try:
            return self.get_networks_with_bridge(bridge)[0]
        except KeyError:
            return None

    def get_pif_by_bridge(self, bridge):
        networks = self.get_networks_with_bridge(bridge)

        if len(networks) == 0:
            raise Error("No matching network \"%s\"" % bridge)

        answer = None
        for network in networks:
            nwrec = self.get_network_record(network)
            for pif in nwrec['PIFs']:
                pifrec = self.get_pif_record(pif)
                if answer:
                    raise Error("Multiple PIFs on host for network %s" % (bridge))
                answer = pif
        if not answer:
            raise Error("No PIF on host for network %s" % (bridge))
        return answer

    def get_pif_record(self, pif):
        if pif in self.__pifs:
            return self.__pifs[pif]
        raise Error("Unknown PIF \"%s\"" % pif)
    def get_all_pifs(self):
        return self.__pifs
    def pif_exists(self, pif):
        return pif in self.__pifs

    def get_management_pif(self):
        """ Returns the management pif on host
        """
        all = self.get_all_pifs()
        for pif in all:
            pifrec = self.get_pif_record(pif)
            if pifrec['management']: return pif
        return None

    def get_network_record(self, network):
        if network in self.__networks:
            return self.__networks[network]
        raise Error("Unknown network \"%s\"" % network)

    def get_bond_record(self, bond):
        if bond in self.__bonds:
            return self.__bonds[bond]
        else:
            return None

    def get_vlan_record(self, vlan):
        if vlan in self.__vlans:
            return self.__vlans[vlan]
        else:
            return None

    def get_pool_record(self):
        if len(self.__pools) > 0:
            return list(self.__pools.values())[0]

#
#
#
PIF_OTHERCONFIG_DEFAULTS = {'gro': 'off', 'lro': 'off'}

def ethtool_settings(oc, defaults = {}):
    settings = []
    if 'ethtool-speed' in oc:
        val = oc['ethtool-speed']
        if val in ["10", "100", "1000"]:
            settings += ['speed', val]
        else:
            log("Invalid value for ethtool-speed = %s. Must be 10|100|1000." % val)
    if 'ethtool-duplex' in oc:
        val = oc['ethtool-duplex']
        if val in ["half", "full"]:
            settings += ['duplex', val]
        else:
            log("Invalid value for ethtool-duplex = %s. Must be half|full." % val)
    if 'ethtool-autoneg' in oc:
        val = oc['ethtool-autoneg']
        if val in ["true", "on"]:
            settings += ['autoneg', 'on']
        elif val in ["false", "off"]:
            settings += ['autoneg', 'off']
        else:
            log("Invalid value for ethtool-autoneg = %s. Must be on|true|off|false." % val)
    offload = []
    for opt in ("rx", "tx", "sg", "tso", "ufo", "gso", "gro", "lro"):
        if "ethtool-" + opt in oc:
            val = oc["ethtool-" + opt]
            if val in ["true", "on"]:
                offload += [opt, 'on']
            elif val in ["false", "off"]:
                offload += [opt, 'off']
            else:
                log("Invalid value for ethtool-%s = %s. Must be on|true|off|false." % (opt, val))
        elif opt in defaults:
            offload += [opt, defaults[opt]]
    return settings,offload

# By default the MTU is taken from the Network.MTU setting for VIF,
# PIF and Bridge. However it is possible to override this by using
# {VIF,PIF,Network}.other-config:mtu.
#
# type parameter is a string describing the object that the oc parameter
# is from. e.g. "PIF", "Network" 
def mtu_setting(nw, type, oc):
    mtu = None

    nwrec = db().get_network_record(nw)
    if 'MTU' in nwrec:
        mtu = nwrec['MTU']
    else:
        mtu = "1500"
        
    if 'mtu' in oc:
        log("Override Network.MTU setting on bridge %s from %s.MTU is %s" % \
            (nwrec['bridge'], type, mtu))
        mtu = oc['mtu']

    if mtu is not None:
        try:
            int(mtu)      # Check that the value is an integer
            return mtu
        except ValueError as x:
            log("Invalid value for mtu = %s" % mtu)

    return None

#
# IP Network Devices -- network devices with IP configuration
#
def pif_ipdev_name(pif):
    """Return the ipdev name associated with pif"""
    pifrec = db().get_pif_record(pif)
    nwrec = db().get_network_record(pifrec['network'])

    if nwrec['bridge']:
        # TODO: sanity check that nwrec['bridgeless'] != 'true'
        return nwrec['bridge']
    else:
        # TODO: sanity check that nwrec['bridgeless'] == 'true'
        return pif_netdev_name(pif)

#
# Bare Network Devices -- network devices without IP configuration
#

def netdev_exists(netdev):
    return os.path.exists(root_prefix() + "/sys/class/net/" + netdev)


def unicode_2to3(string):
    if sys.version_info < (3,):
        return string.encode()
    return string


def pif_netdev_name(pif):
    """Get the netdev name for a PIF."""

    pifrec = db().get_pif_record(pif)

    if pif_is_vlan(pif):
        return unicode_2to3("%(device)s.%(VLAN)s" % pifrec)
    else:
        return unicode_2to3(pifrec['device'])

#
# Bridges
#

def pif_is_bridged(pif):
    pifrec = db().get_pif_record(pif)
    nwrec = db().get_network_record(pifrec['network'])

    if nwrec['bridge']:
        # TODO: sanity check that nwrec['bridgeless'] != 'true'
        return True
    else:
        # TODO: sanity check that nwrec['bridgeless'] == 'true'
        return False

def pif_bridge_name(pif):
    """Return the bridge name of a pif.

    PIF must be a bridged PIF."""
    pifrec = db().get_pif_record(pif)

    nwrec = db().get_network_record(pifrec['network'])

    if nwrec['bridge']:
        return nwrec['bridge']
    else:
        raise Error("PIF %(uuid)s does not have a bridge name" % pifrec)

#
# Bonded PIFs
#
def pif_is_bond(pif):
    pifrec = db().get_pif_record(pif)

    return len(pifrec['bond_master_of']) > 0

def pif_get_bond_masters(pif):
    """Returns a list of PIFs which are bond masters of this PIF"""

    pifrec = db().get_pif_record(pif)

    bso = pifrec['bond_slave_of']

    # bond-slave-of is currently a single reference but in principle a
    # PIF could be a member of several bonds which are not
    # concurrently attached. Be robust to this possibility.
    if not bso or bso == "OpaqueRef:NULL":
        bso = []
    elif not type(bso) == list:
        bso = [bso]

    bondrecs = [db().get_bond_record(bond) for bond in bso]
    bondrecs = [rec for rec in bondrecs if rec]

    return [bond['master'] for bond in bondrecs]

def pif_get_bond_slaves(pif):
    """Returns a list of PIFs which make up the given bonded pif."""

    pifrec = db().get_pif_record(pif)

    bmo = pifrec['bond_master_of']
    if len(bmo) > 1:
        raise Error("Bond-master-of contains too many elements")

    if len(bmo) == 0:
        return []

    bondrec = db().get_bond_record(bmo[0])
    if not bondrec:
        raise Error("No bond record for bond master PIF")

    return bondrec['slaves']

#
# VLAN PIFs
#

def pif_is_vlan(pif):
    return db().get_pif_record(pif)['VLAN'] != '-1'

def pif_get_vlan_slave(pif):
    """Find the PIF which is the VLAN slave of pif.

Returns the 'physical' PIF underneath the a VLAN PIF @pif."""

    pifrec = db().get_pif_record(pif)

    vlan = pifrec['VLAN_master_of']
    if not vlan or vlan == "OpaqueRef:NULL":
        raise Error("PIF is not a VLAN master")

    vlanrec = db().get_vlan_record(vlan)
    if not vlanrec:
        raise Error("No VLAN record found for PIF")

    return vlanrec['tagged_PIF']

def pif_get_vlan_masters(pif):
    """Returns a list of PIFs which are VLANs on top of the given pif."""

    pifrec = db().get_pif_record(pif)
    vlans = [db().get_vlan_record(v) for v in pifrec['VLAN_slave_of']]
    return [v['untagged_PIF'] for v in vlans if v and db().pif_exists(v['untagged_PIF'])]

#
# Tunnel PIFs
#
def pif_is_tunnel(pif):
    return len(db().get_pif_record(pif)['tunnel_access_PIF_of']) > 0

#
# Datapath base class
#

class Datapath(object):
    """Object encapsulating the actions necessary to (de)configure the
       datapath for a given PIF. Does not include configuration of the
       IP address on the ipdev.
    """
    
    def __init__(self, pif):
        self._pif = pif

    @classmethod
    def rewrite(cls):
        """Class method called when write action is called. Can be used
           to update any backend specific configuration."""
        pass

    def configure_ipdev(self, cfg):
        """Write ifcfg TYPE field for an IPdev, plus any type specific
           fields to cfg
        """
        raise NotImplementedError        

    def preconfigure(self, parent):
        """Prepare datapath configuration for PIF, but do not actually
           apply any changes.

           Any configuration files should be attached to parent.
        """
        raise NotImplementedError
    
    def bring_down_existing(self):
        """Tear down any existing network device configuration which
           needs to be undone in order to bring this PIF up.
        """
        raise NotImplementedError

    def configure(self):
        """Apply the configuration prepared in the preconfigure stage.

           Should assume any configuration files changed attached in
           the preconfigure stage are applied and bring up the
           necessary devices to provide the datapath for the
           PIF.

           Should not bring up the IPdev.
        """
        raise NotImplementedError
    
    def post(self):
        """Called after the IPdev has been brought up.

           Should do any final setup, including reinstating any
           devices which were taken down in the bring_down_existing
           hook.
        """
        raise NotImplementedError

    def bring_down(self):
        """Tear down and deconfigure the datapath. Should assume the
           IPdev has already been brought down.
        """
        raise NotImplementedError
        
def DatapathFactory():
    # XXX Need a datapath object for bridgeless PIFs

    try:
        network_conf = open(root_prefix() + "/etc/xensource/network.conf", 'r')
        network_backend = network_conf.readline().strip()
        network_conf.close()                
    except Exception as e:
        raise Error("failed to determine network backend:" + e)
    
    if network_backend == "bridge":
        from InterfaceReconfigureBridge import DatapathBridge
        return DatapathBridge
    elif network_backend in ["openvswitch", "vswitch"]:
        from InterfaceReconfigureVswitch import DatapathVswitch
        return DatapathVswitch
    else:
        raise Error("unknown network backend %s" % network_backend)



#! /usr/bin/env python3

# Copyright (c) 2009, 2010, 2011, 2012, 2015, 2017 Nicira, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at:
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import socket
import struct


def pack_ethaddr(ea):
    octets = ea.split(':')
    assert len(octets) == 6
    return b''.join([struct.pack('B', int(octet, 16)) for octet in octets])


def output(attrs):
    # Compose flow.

    flow = {}
    flow['DL_SRC'] = "00:02:e3:0f:80:a4"
    flow['DL_DST'] = "00:1a:92:40:ac:05"
    flow['NW_PROTO'] = 0
    flow['NW_TOS'] = 0
    flow['NW_SRC'] = '0.0.0.0'
    flow['NW_DST'] = '0.0.0.0'
    flow['TP_SRC'] = 0
    flow['TP_DST'] = 0
    if 'DL_VLAN' in attrs:
        flow['DL_VLAN'] = {'none': 0xffff,
                           'zero': 0,
                           'nonzero': 0x0123}[attrs['DL_VLAN']]
    else:
        flow['DL_VLAN'] = 0xffff  # OFP_VLAN_NONE
    if attrs['DL_HEADER'] == '802.2':
        flow['DL_TYPE'] = 0x5ff  # OFP_DL_TYPE_NOT_ETH_TYPE
    elif attrs['DL_TYPE'] == 'ip':
        flow['DL_TYPE'] = 0x0800  # ETH_TYPE_IP
        flow['NW_SRC'] = '10.0.2.15'
        flow['NW_DST'] = '192.168.1.20'
        flow['NW_TOS'] = 44
        if attrs['TP_PROTO'] == 'other':
            flow['NW_PROTO'] = 42
        elif attrs['TP_PROTO'] in ('TCP', 'TCP+options'):
            flow['NW_PROTO'] = 6  # IPPROTO_TCP
            flow['TP_SRC'] = 6667
            flow['TP_DST'] = 9998
        elif attrs['TP_PROTO'] == 'UDP':
            flow['NW_PROTO'] = 17  # IPPROTO_UDP
            flow['TP_SRC'] = 1112
            flow['TP_DST'] = 2223
        elif attrs['TP_PROTO'] == 'ICMP':
            flow['NW_PROTO'] = 1  # IPPROTO_ICMP
            flow['TP_SRC'] = 8    # echo request
            flow['TP_DST'] = 0    # code
        else:
            assert False
        if attrs['IP_FRAGMENT'] not in ('no', 'first'):
            flow['TP_SRC'] = flow['TP_DST'] = 0
    elif attrs['DL_TYPE'] == 'non-ip':
        flow['DL_TYPE'] = 0x5678
    else:
        assert False

    # Compose packet
    packet = b''
    wildcards = 1 << 5 | 1 << 6 | 1 << 7 | 32 << 8 | 32 << 14 | 1 << 21

    packet += pack_ethaddr(flow['DL_DST'])
    packet += pack_ethaddr(flow['DL_SRC'])
    if flow['DL_VLAN'] != 0xffff:
        packet += struct.pack('>HH', 0x8100, flow['DL_VLAN'])
    len_ofs = len(packet)
    if attrs['DL_HEADER'].startswith('802.2'):
        packet += struct.pack('>H', 0)
    if attrs['DL_HEADER'] == '802.2':
        packet += struct.pack('BBB', 0x42, 0x42, 0x03)  # LLC for 802.1D STP
    else:
        if attrs['DL_HEADER'] == '802.2+SNAP':
            packet += struct.pack('BBB', 0xaa, 0xaa, 0x03)  # LLC for SNAP
            packet += struct.pack('BBB', 0, 0, 0)           # SNAP OUI
        packet += struct.pack('>H', flow['DL_TYPE'])
        if attrs['DL_TYPE'] == 'ip':
            ip = struct.pack('>BBHHHBBHLL',
                             (4 << 4) | 5,      # version, hdrlen
                             flow['NW_TOS'],    # type of service
                             0,                 # total length, filled in later
                             65432,             # id
                             0,                 # frag offset
                             64,                # ttl
                             flow['NW_PROTO'],  # protocol
                             0,                 # checksum
                             0x0a00020f,        # source
                             0xc0a80114)        # dest
            wildcards &= ~(1 << 5 | 63 << 8 | 63 << 14 | 1 << 21)
            if attrs['IP_OPTIONS'] == 'yes':
                ip = struct.pack('B', (4 << 4) | 8) + ip[1:]
                ip += struct.pack('>BBHHHBBBx',
                                  130,       # type
                                  11,        # length
                                  0x6bc5,    # top secret
                                  0xabcd,
                                  0x1234,
                                  1,
                                  2,
                                  3)
            if attrs['IP_FRAGMENT'] != 'no':
                frag_map = {'first': 0x2000,   # more frags, ofs 0
                            'middle': 0x2111,  # more frags, ofs 0x888
                            'last': 0x0222}    # last frag, ofs 0x1110
                ip = (ip[:6]
                      + struct.pack('>H', frag_map[attrs['IP_FRAGMENT']])
                      + ip[8:])
            if attrs['IP_FRAGMENT'] in ('no', 'first'):
                if attrs['TP_PROTO'].startswith('TCP'):
                    tcp = struct.pack('>HHLLHHHH',
                                      flow['TP_SRC'],  # source port
                                      flow['TP_DST'],  # dest port
                                      87123455,        # seqno
                                      712378912,       # ackno
                                      (5 << 12) | 0x02 | 0x10,
                                           # hdrlen, SYN, ACK
                                      5823,   # window size
                                      18923,  # checksum
                                      12893)  # urgent pointer
                    if attrs['TP_PROTO'] == 'TCP+options':
                        tcp = (tcp[:12]
                               + struct.pack('>H', (6 << 12) | 0x02 | 0x10)
                               + tcp[14:])
                        tcp += struct.pack('>BBH', 2, 4, 1975)  # MSS option
                    tcp += b'payload'
                    ip += tcp
                    wildcards &= ~(1 << 6 | 1 << 7)
                elif attrs['TP_PROTO'] == 'UDP':
                    udp_len = 15
                    udp = struct.pack('>HHHH',
                                      flow['TP_SRC'],
                                      flow['TP_DST'],
                                      udp_len, 0)
                    while len(udp) < udp_len:
                        udp += struct.pack('B', udp_len)
                    ip += udp
                    wildcards &= ~(1 << 6 | 1 << 7)
                elif attrs['TP_PROTO'] == 'ICMP':
                    ip += struct.pack('>BBHHH',
                                      8,        # echo request
                                      0,        # code
                                      0,        # checksum
                                      736,      # identifier
                                      931)      # sequence number
                    wildcards &= ~(1 << 6 | 1 << 7)
                elif attrs['TP_PROTO'] == 'other':
                    ip += b'other header'
                else:
                    assert False
            ip = ip[:2] + struct.pack('>H', len(ip)) + ip[4:]
            packet += ip
    if attrs['DL_HEADER'].startswith('802.2'):
        packet_len = len(packet) - 14
        if flow['DL_VLAN'] != 0xffff:
            packet_len -= 4
        packet = (packet[:len_ofs]
                  + struct.pack('>H', packet_len)
                  + packet[len_ofs + 2:])

    print(' '.join(['%s=%s' % (k, v) for k, v in attrs.items()]))
    print(' '.join(['%s=%s' % (k, v) for k, v in flow.items()]))
    print()

    flows.write(struct.pack('>LH',
                            wildcards,  # wildcards
                            1))         # in_port
    flows.write(pack_ethaddr(flow['DL_SRC']))
    flows.write(pack_ethaddr(flow['DL_DST']))
    flows.write(struct.pack('>HBxHBBxx',
                            flow['DL_VLAN'],
                            0,  # DL_VLAN_PCP
                            flow['DL_TYPE'],
                            flow['NW_TOS'],
                            flow['NW_PROTO']))
    flows.write(socket.inet_aton(flow['NW_SRC']))
    flows.write(socket.inet_aton(flow['NW_DST']))
    flows.write(struct.pack('>HH', flow['TP_SRC'], flow['TP_DST']))

    packets.write(struct.pack('>LLLL',
                              0,                # timestamp seconds
                              0,                # timestamp microseconds
                              len(packet),      # bytes saved
                              len(packet)))     # total length
    packets.write(packet)


flows = open('flows', 'wb')
packets = open('pcap', 'wb')

# Print pcap file header.
packets.write(struct.pack('>LHHLLLL',
                          0xa1b2c3d4,  # magic number
                          2,           # major version
                          4,           # minor version
                          0,           # time zone offset
                          0,           # time stamp accuracy
                          1518,        # snaplen
                          1))          # Ethernet

output({'DL_HEADER': '802.2'})

for dl_header in ('802.2+SNAP', 'Ethernet'):
    a = {'DL_HEADER': dl_header}
    for dl_vlan in ('none', 'zero', 'nonzero'):
        b = a.copy()
        b['DL_VLAN'] = dl_vlan

        # Non-IP case.
        c = b.copy()
        c['DL_TYPE'] = 'non-ip'
        output(c)

        for ip_options in ('no', 'yes'):
            c = b.copy()
            c['DL_TYPE'] = 'ip'
            c['IP_OPTIONS'] = ip_options
            for ip_fragment in ('no', 'first', 'middle', 'last'):
                d = c.copy()
                d['IP_FRAGMENT'] = ip_fragment
                for tp_proto in ('TCP', 'TCP+options', 'UDP', 'ICMP', 'other'):
                    e = d.copy()
                    e['TP_PROTO'] = tp_proto
                    output(e)



# Copyright (c) 2009,2010,2011,2012,2013 Nicira, Inc.
# Copyright (c) 2007-2011 Citrix Systems Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 only.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

from XSConsoleLog import *

import os
import socket
import subprocess

vsctl="/usr/bin/ovs-vsctl"

if __name__ == "__main__":
    raise Exception("This script is a plugin for xsconsole and cannot run independently")

from XSConsoleStandard import *

class VSwitchService:
    service = {}

    def __init__(self, name, processname=None):
        self.name = name
        self.processname = processname
        if self.processname == None:
            self.processname = name

    def version(self):
        try:
            output = ShellPipe(["service", self.name, "version"]).Stdout()
        except StandardError as e:
            XSLogError("vswitch version retrieval error: " + str(e))
            return "<unknown>"
        for line in output:
            if self.processname in line:
                return line.split()[-1]
        return "<unknown>"

    def status(self):
        try:
            output = ShellPipe(["service", self.name, "status"]).Stdout()
        except StandardError as e:
            XSLogError("vswitch status retrieval error: " + str(e))
            return "<unknown>"
        if len(output) == 0:
            return "<unknown>"
        for line in output:
            if self.processname not in line:
                continue
            elif "running" in line:
                return "Running"
            elif "stop" in line:
                return "Stopped"
            else:
                return "<unknown>"
        return "<unknown>"

    def restart(self):
        try:
            ShellPipe(["service", self.name, "restart"]).Call()
        except StandardError as e:
            XSLogError("vswitch restart error: " + str(e))

    @classmethod
    def Inst(cls, name, processname=None):
        key = name
        if processname != None:
            key = key + "-" + processname
        if name not in cls.service:
            cls.service[key] = VSwitchService(name, processname)
        return cls.service[key]

class VSwitchConfig:

    @staticmethod
    def Get(action):
        try:
            arg = [vsctl, "-vconsole:off"] + action.split()
            output = ShellPipe(arg).Stdout()
        except StandardError as e:
            XSLogError("config retrieval error: " + str(e))
            return "<unknown>"

        if len(output) == 0:
            output = ""
        else:
            output = output[0].strip()
        return output


class VSwitchControllerDialogue(Dialogue):
    def __init__(self):
        Dialogue.__init__(self)
        data=Data.Inst()

        self.hostsInPool = 0
        self.hostsUpdated = 0
        self.xs_version = data.host.software_version.product_version('')
        pool = data.GetPoolForThisHost()
        if pool is not None:
            self.controller = pool.get("vswitch_controller", "")
        else:
            self.controller = ""

        choiceDefs = [
            ChoiceDef(Lang("Set pool-wide controller"),
                      lambda: self.getController()),
            ChoiceDef(Lang("Delete pool-wide controller"),
                      lambda: self.deleteController()),
            ChoiceDef(Lang("Resync server controller config"),
                      lambda: self.syncController()),
#             ChoiceDef(Lang("Restart ovs-vswitchd"),
#                       lambda: self.restartService("vswitch")),
            ]
        self.menu = Menu(self, None, Lang("Configure Open vSwitch"), choiceDefs)

        self.ChangeState("INITIAL")

    def BuildPane(self):
        pane = self.NewPane(DialoguePane(self.parent))
        pane.TitleSet(Lang("Configure Open vSwitch"))
        pane.AddBox()

    def ChangeState(self, inState):
        self.state = inState
        self.BuildPane()
        self.UpdateFields()

    def UpdateFields(self):
        self.Pane().ResetPosition()
        getattr(self, "UpdateFields" + self.state)() # Dispatch method named 'UpdateFields'+self.state

    def UpdateFieldsINITIAL(self):
        pane = self.Pane()
        pane.AddTitleField(Lang("Select an action"))
        pane.AddMenuField(self.menu)
        pane.AddKeyHelpField( { Lang("<Enter>") : Lang("OK"), Lang("<Esc>") : Lang("Cancel") } )

    def UpdateFieldsGETCONTROLLER(self):
        pane = self.Pane()
        pane.ResetFields()

        pane.AddTitleField(Lang("Enter IP address of controller"))
        pane.AddInputField(Lang("Address", 16), self.controller, "address")
        pane.AddKeyHelpField( { Lang("<Enter>") : Lang("OK"), Lang("<Esc>") : Lang("Exit") } )
        if pane.CurrentInput() is None:
            pane.InputIndexSet(0)

    def HandleKey(self, inKey):
        handled = False
        if hasattr(self, "HandleKey" + self.state):
            handled = getattr(self, "HandleKey" + self.state)(inKey)
        if not handled and inKey == 'KEY_ESCAPE':
            Layout.Inst().PopDialogue()
            handled = True
        return handled

    def HandleKeyINITIAL(self, inKey):
        return self.menu.HandleKey(inKey)

    def HandleKeyGETCONTROLLER(self, inKey):
        pane = self.Pane()
        if pane.CurrentInput() is None:
            pane.InputIndexSet(0)
        if inKey == 'KEY_ENTER':
            inputValues = pane.GetFieldValues()
            self.controller = inputValues['address']
            Layout.Inst().PopDialogue()

            # Make sure the controller is specified as a valid dotted quad
            try:
                socket.inet_aton(self.controller)
            except socket.error:
                Layout.Inst().PushDialogue(InfoDialogue(Lang("Please enter in dotted quad format")))
                return True

            Layout.Inst().TransientBanner(Lang("Setting controller..."))
            try:
                self.SetController(self.controller)
                Layout.Inst().PushDialogue(InfoDialogue(Lang("Setting controller successful")))
            except Exception as e:
                Layout.Inst().PushDialogue(InfoDialogue(Lang("Setting controller failed")))

            self.ChangeState("INITIAL")
            return True
        else:
            return pane.CurrentInput().HandleKey(inKey)

    def restartService(self, name):
        s = VSwitchService.Inst(name)
        s.restart()
        Layout.Inst().PopDialogue()

    def getController(self):
        self.ChangeState("GETCONTROLLER")
        self.Pane().InputIndexSet(0)

    def deleteController(self):
        self.controller = ""
        Layout.Inst().PopDialogue()
        Layout.Inst().TransientBanner(Lang("Deleting controller..."))
        try:
            self.SetController(None)
            Layout.Inst().PushDialogue(InfoDialogue(Lang("Controller deletion successful")))
        except Exception as e:
            Layout.Inst().PushDialogue(InfoDialogue(Lang("Controller deletion failed")))

    def syncController(self):
        Layout.Inst().PopDialogue()
        Layout.Inst().TransientBanner(Lang("Resyncing controller setting..."))
        try:
            Task.Sync(lambda s: self._updateThisServer(s))
            Layout.Inst().PushDialogue(InfoDialogue(Lang("Resyncing controller config successful")))
        except Exception as e:
            Layout.Inst().PushDialogue(InfoDialogue(Lang("Resyncing controller config failed")))

    def SetController(self, ip):
        self.hostsInPool = 0
        self.hostsUpdated = 0
        Task.Sync(lambda s: self._modifyPoolConfig(s, ip or ""))
        # Should be done asynchronously, maybe with an external script?
        Task.Sync(lambda s: self._updateActiveServers(s))

    def _modifyPoolConfig(self, session, value):
        """Modify pool configuration.

        If value == "" then delete configuration, otherwise set to value.
        """
        pools = session.xenapi.pool.get_all()
        # We assume there is only ever one pool...
        if len(pools) == 0:
            XSLogFatal(Lang("No pool found for host."))
            return
        if len(pools) > 1:
            XSLogFatal(Lang("More than one pool for host."))
            return
        session.xenapi.pool.set_vswitch_controller(value)
        Data.Inst().Update()

    def _updateActiveServers(self, session):
        hosts = session.xenapi.host.get_all()
        self.hostsUpdated = 0
        self.hostsInPool = len(hosts)
        self.UpdateFields()
        for host in hosts:
            Layout.Inst().TransientBanner("Updating host %d out of %d" 
                    % (self.hostsUpdated + 1, self.hostsInPool))
            session.xenapi.host.call_plugin(host, "openvswitch-cfg-update", "update", {})
            self.hostsUpdated = self.hostsUpdated + 1

    def _updateThisServer(self, session):
        data = Data.Inst()
        host = data.host.opaqueref()
        session.xenapi.host.call_plugin(host, "openvswitch-cfg-update", "update", {})


class XSFeatureVSwitch:

    @classmethod
    def StatusUpdateHandler(cls, inPane):
        data = Data.Inst()
        xs_version = data.host.software_version.product_version('')

        inPane.AddTitleField(Lang("Open vSwitch"))

        inPane.NewLine()

        inPane.AddStatusField(Lang("Version", 20),
                              VSwitchService.Inst("openvswitch", "ovs-vswitchd").version())

        inPane.NewLine()

        pool = data.GetPoolForThisHost()
        if pool is not None:
            dbController = pool.get("vswitch_controller", "")
        else:
            dbController = ""

        if dbController == "":
            dbController = Lang("<None>")
        inPane.AddStatusField(Lang("Controller (config)", 20), dbController)
        controller = VSwitchConfig.Get("get-manager")

        if controller == "":
            controller = Lang("<None>")
        elif controller[0:4] == "ssl:":
            controller = controller.split(':')[1]
        inPane.AddStatusField(Lang("Controller (in-use)", 20), controller)

        inPane.NewLine()
        inPane.AddStatusField(Lang("ovs-vswitchd status", 20),
                              VSwitchService.Inst("openvswitch", "ovs-vswitchd").status())
        inPane.AddStatusField(Lang("ovsdb-server status", 20),
                              VSwitchService.Inst("openvswitch", "ovsdb-server").status())

        inPane.AddKeyHelpField( {
            Lang("<Enter>") : Lang("Reconfigure"),
            Lang("<F5>") : Lang("Refresh")
        })

    @classmethod
    def ActivateHandler(cls):
        DialogueUtils.AuthenticatedOnly(lambda: Layout.Inst().PushDialogue(VSwitchControllerDialogue()))

    def Register(self):
        Importer.RegisterNamedPlugIn(
            self,
            'VSwitch', # Key of this plugin for replacement, etc.
            {
                'menuname' : 'MENU_NETWORK',
                'menupriority' : 800,
                'menutext' : Lang('Open vSwitch') ,
                'statusupdatehandler' : self.StatusUpdateHandler,
                'activatehandler' : self.ActivateHandler
            }
        )

# Register this plugin when module is imported, IFF vswitchd is running
if os.path.exists('/var/run/openvswitch/ovs-vswitchd.pid'):
    XSFeatureVSwitch().Register()



# Copyright (c) 2009, 2010 Nicira, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at:
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import sys
import time

try:
    import ctypes

    LIBRT = 'librt.so.1'
    clock_gettime_name = 'clock_gettime'

    if sys.platform.startswith("linux"):
        CLOCK_MONOTONIC = 1
        time_t = ctypes.c_long
    elif sys.platform.startswith("netbsd"):
        # NetBSD uses function renaming for ABI versioning.  While the proper
        # way to get the appropriate version is of course "#include <time.h>",
        # it is difficult with ctypes.  The following is appropriate for
        # recent versions of NetBSD, including NetBSD-6.
        LIBRT = 'libc.so.12'
        clock_gettime_name = '__clock_gettime50'
        CLOCK_MONOTONIC = 3
        time_t = ctypes.c_int64
    elif sys.platform.startswith("freebsd"):
        CLOCK_MONOTONIC = 4
        time_t = ctypes.c_int64
    else:
        raise Exception

    class timespec(ctypes.Structure):
        _fields_ = [
            ('tv_sec', time_t),
            ('tv_nsec', ctypes.c_long),
        ]

    librt = ctypes.CDLL(LIBRT)
    clock_gettime = getattr(librt, clock_gettime_name)
    clock_gettime.argtypes = [ctypes.c_int, ctypes.POINTER(timespec)]
except:
    # Librt shared library could not be loaded
    librt = None


def monotonic():
    if not librt:
        return time.time()

    t = timespec()
    if clock_gettime(CLOCK_MONOTONIC, ctypes.pointer(t)) == 0:
        return t.tv_sec + t.tv_nsec * 1e-9
    # Kernel does not support CLOCK_MONOTONIC
    return time.time()


# Use time.monotonic() if Python version >= 3.3
if not hasattr(time, 'monotonic'):
    time.monotonic = monotonic


def msec():
    """ Returns the system's monotonic time if possible, otherwise returns the
    current time as the amount of time since the epoch, in milliseconds, as a
    float."""
    return time.monotonic() * 1000.0


def postfork():
    # Just a stub for now
    pass



# Copyright (c) 2011, 2012 Nicira, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at:
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
rpcserver is an XML RPC server that allows RPC client to initiate tests
"""

import sys

import xmlrpc.client

from twisted.internet import reactor
from twisted.internet.error import CannotListenError
from twisted.web import server
from twisted.web import xmlrpc

from . import tcp, udp, util, vswitch


class TestArena(xmlrpc.XMLRPC):
    """
    This class contains all the functions that ovs-test client will call
    remotely. The caller is responsible to use designated handleIds
    for designated methods (e.g. do not mix UDP and TCP handles).
    """

    def __init__(self):
        xmlrpc.XMLRPC.__init__(self, allowNone=True)
        self.handle_id = 1
        self.handle_map = {}
        self.bridges = set()
        self.pbridges = set()
        self.ports = set()
        self.request = None

    def __acquire_handle(self, value):
        """
        Allocates new handle and assigns value object to it
        """
        handle = self.handle_id
        self.handle_map[handle] = value
        self.handle_id += 1
        return handle

    def __get_handle_resources(self, handle):
        """
        Return resources that were assigned to handle
        """
        return self.handle_map[handle]

    def __delete_handle(self, handle):
        """
        Releases handle from handle_map
        """
        del self.handle_map[handle]

    def cleanup(self):
        """
        Delete all remaining bridges and ports if ovs-test client did not had
        a chance to remove them. It is necessary to call this function if
        ovs-test server is abruptly terminated when doing the tests.
        """
        for port in self.ports:
            # Remove ports that were added to existing bridges
            vswitch.ovs_vsctl_del_port_from_bridge(port)

        for bridge in self.bridges:
            # Remove bridges that were added for L3 tests
            vswitch.ovs_vsctl_del_bridge(bridge)

        for pbridge in self.pbridges:
            # Remove bridges that were added for VLAN tests
            vswitch.ovs_vsctl_del_pbridge(pbridge[0], pbridge[1])

    def render(self, request):
        """
        This method overrides the original XMLRPC.render method so that it
        would be possible to get the XML RPC client IP address from the
        request object.
        """
        self.request = request
        return xmlrpc.XMLRPC.render(self, request)

    def xmlrpc_get_my_address(self):
        """
        Returns the RPC client's IP address.
        """
        return self.request.getClientIP()

    def xmlrpc_get_my_address_from(self, his_ip, his_port):
        """
        Returns the ovs-test server IP address that the other ovs-test server
        with the given ip will see.
        """
        server1 = xmlrpc.client.Server("http://%s:%u/" % (his_ip, his_port))
        return server1.get_my_address()

    def xmlrpc_create_udp_listener(self, port):
        """
        Creates a UDP listener that will receive packets from UDP sender
        """
        try:
            listener = udp.UdpListener()
            reactor.listenUDP(port, listener)
            handle_id = self.__acquire_handle(listener)
        except CannotListenError:
            return -1
        return handle_id

    def xmlrpc_create_udp_sender(self, host, count, size, duration):
        """
        Send UDP datagrams to UDP listener
        """
        sender = udp.UdpSender(tuple(host), count, size, duration)
        reactor.listenUDP(0, sender)
        handle_id = self.__acquire_handle(sender)
        return handle_id

    def xmlrpc_get_udp_listener_results(self, handle):
        """
        Returns number of datagrams that were received
        """
        listener = self.__get_handle_resources(handle)
        return listener.getResults()

    def xmlrpc_get_udp_sender_results(self, handle):
        """
        Returns number of datagrams that were sent
        """
        sender = self.__get_handle_resources(handle)
        return sender.getResults()

    def xmlrpc_close_udp_listener(self, handle):
        """
        Releases UdpListener and all its resources
        """
        listener = self.__get_handle_resources(handle)
        listener.transport.stopListening()
        self.__delete_handle(handle)
        return 0

    def xmlrpc_close_udp_sender(self, handle):
        """
        Releases UdpSender and all its resources
        """
        sender = self.__get_handle_resources(handle)
        sender.transport.stopListening()
        self.__delete_handle(handle)
        return 0

    def xmlrpc_create_tcp_listener(self, port):
        """
        Creates a TcpListener that will accept connection from TcpSender
        """
        try:
            listener = tcp.TcpListenerFactory()
            port = reactor.listenTCP(port, listener)
            handle_id = self.__acquire_handle((listener, port))
            return handle_id
        except CannotListenError:
            return -1

    def xmlrpc_create_tcp_sender(self, his_ip, his_port, duration):
        """
        Creates a TcpSender that will connect to TcpListener
        """
        sender = tcp.TcpSenderFactory(duration)
        connector = reactor.connectTCP(his_ip, his_port, sender)
        handle_id = self.__acquire_handle((sender, connector))
        return handle_id

    def xmlrpc_get_tcp_listener_results(self, handle):
        """
        Returns number of bytes received
        """
        (listener, _) = self.__get_handle_resources(handle)
        return listener.getResults()

    def xmlrpc_get_tcp_sender_results(self, handle):
        """
        Returns number of bytes sent
        """
        (sender, _) = self.__get_handle_resources(handle)
        return sender.getResults()

    def xmlrpc_close_tcp_listener(self, handle):
        """
        Releases TcpListener and all its resources
        """
        try:
            (_, port) = self.__get_handle_resources(handle)
            port.loseConnection()
            self.__delete_handle(handle)
        except KeyError:
            return -1
        return 0

    def xmlrpc_close_tcp_sender(self, handle):
        """
        Releases TcpSender and all its resources
        """
        try:
            (_, connector) = self.__get_handle_resources(handle)
            connector.disconnect()
            self.__delete_handle(handle)
        except KeyError:
            return -1
        return 0

    def xmlrpc_create_test_bridge(self, bridge, iface):
        """
        This function creates a physical bridge from iface. It moves the
        IP configuration from the physical interface to the bridge.
        """
        ret = vswitch.ovs_vsctl_add_bridge(bridge)
        if ret == 0:
            self.pbridges.add((bridge, iface))
            util.interface_up(bridge)
            (ip_addr, mask) = util.interface_get_ip(iface)
            util.interface_assign_ip(bridge, ip_addr, mask)
            util.interface_up(bridge)
            util.move_routes(iface, bridge)
            util.interface_remove_ip(iface, ip_addr, mask)
            ret = vswitch.ovs_vsctl_add_port_to_bridge(bridge, iface)
            if ret == 0:
                self.ports.add(iface)
            else:
                util.interface_assign_ip(iface, ip_addr, mask)
                util.interface_up(iface)
                util.move_routes(bridge, iface)
                vswitch.ovs_vsctl_del_bridge(bridge)

        return ret

    def xmlrpc_del_test_bridge(self, bridge, iface):
        """
        This function deletes the test bridge and moves its IP configuration
        back to the physical interface.
        """
        ret = vswitch.ovs_vsctl_del_pbridge(bridge, iface)
        self.pbridges.discard((bridge, iface))
        return ret

    def xmlrpc_get_iface_from_bridge(self, brname):
        """
        Tries to figure out physical interface from bridge.
        """
        return vswitch.ovs_get_physical_interface(brname)

    def xmlrpc_create_bridge(self, brname):
        """
        Creates an OVS bridge.
        """
        ret = vswitch.ovs_vsctl_add_bridge(brname)
        if ret == 0:
            self.bridges.add(brname)
        return ret

    def xmlrpc_del_bridge(self, brname):
        """
        Deletes an OVS bridge.
        """
        ret = vswitch.ovs_vsctl_del_bridge(brname)
        if ret == 0:
            self.bridges.discard(brname)
        return ret

    def xmlrpc_is_ovs_bridge(self, bridge):
        """
        This function verifies whether given interface is an ovs bridge.
        """
        return vswitch.ovs_vsctl_is_ovs_bridge(bridge)

    def xmlrpc_add_port_to_bridge(self, bridge, port):
        """
        Adds a port to the OVS bridge.
        """
        ret = vswitch.ovs_vsctl_add_port_to_bridge(bridge, port)
        if ret == 0:
            self.ports.add(port)
        return ret

    def xmlrpc_del_port_from_bridge(self, port):
        """
        Removes a port from OVS bridge.
        """
        ret = vswitch.ovs_vsctl_del_port_from_bridge(port)
        if ret == 0:
            self.ports.discard(port)
        return ret

    def xmlrpc_ovs_vsctl_set(self, table, record, column, key, value):
        """
        This function allows to alter OVS database.
        """
        return vswitch.ovs_vsctl_set(table, record, column, key, value)

    def xmlrpc_interface_up(self, iface):
        """
        This function brings up given interface.
        """
        return util.interface_up(iface)

    def xmlrpc_interface_assign_ip(self, iface, ip_address, mask):
        """
        This function allows to assing ip address to the given interface.
        """
        return util.interface_assign_ip(iface, ip_address, mask)

    def xmlrpc_interface_remove_ip(self, iface, ip_address, mask):
        """
        This function allows to assing ip address to the given interface.
        """
        return util.interface_remove_ip(iface, ip_address, mask)

    def xmlrpc_get_interface(self, address):
        """
        Finds first interface that has given address
        """
        return util.get_interface(address)

    def xmlrpc_get_interface_mtu(self, iface):
        """
        Returns MTU of the given interface
        """
        return util.get_interface_mtu(iface)

    def xmlrpc_uname(self):
        """
        Return information about running kernel
        """
        return util.uname()

    def xmlrpc_get_driver(self, iface):
        """
        Returns driver version
        """
        return util.get_driver(iface)

    def xmlrpc_get_interface_from_routing_decision(self, ip):
        """
        Returns driver version
        """
        return util.get_interface_from_routing_decision(ip)


def start_rpc_server(port):
    """
    This function creates a RPC server and adds it to the Twisted Reactor.
    """
    rpc_server = TestArena()
    reactor.listenTCP(port, server.Site(rpc_server))
    try:
        print("Starting RPC server\n")
        sys.stdout.flush()
        # If this server was started from ovs-test client then we must flush
        # STDOUT so that client would know that server is ready to accept
        # XML RPC connections.
        reactor.run()
    finally:
        rpc_server.cleanup()



#!/usr/bin/python3

import sys

from scapy.all import RandMAC, RandIP, PcapWriter, RandIP6, RandShort, fuzz
from scapy.all import IPv6, Dot1Q, IP, Ether, UDP, TCP

path = str(sys.argv[1]) + "/pcap/fuzzy.pcap"
pktdump = PcapWriter(path, append=False, sync=True)

for i in range(0, 2000):

    # Generate random protocol bases, use a fuzz() over the combined packet
    # for full fuzzing.
    eth = Ether(src=RandMAC(), dst=RandMAC())
    vlan = Dot1Q()
    ipv4 = IP(src=RandIP(), dst=RandIP())
    ipv6 = IPv6(src=RandIP6(), dst=RandIP6())
    udp = UDP(dport=RandShort(), sport=RandShort())
    tcp = TCP(dport=RandShort(), sport=RandShort())

    # IPv4 packets with fuzzing
    pktdump.write(fuzz(eth / ipv4 / udp))
    pktdump.write(fuzz(eth / ipv4 / tcp))
    pktdump.write(fuzz(eth / vlan / ipv4 / udp))
    pktdump.write(fuzz(eth / vlan / ipv4 / tcp))

    # IPv6 packets with fuzzing
    pktdump.write(fuzz(eth / ipv6 / udp))
    pktdump.write(fuzz(eth / ipv6 / tcp))
    pktdump.write(fuzz(eth / vlan / ipv6 / udp))
    pktdump.write(fuzz(eth / vlan / ipv6 / tcp))



# Copyright (c) 2010, 2011, 2012 Nicira, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at:
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import absolute_import

import functools
import json
import re
import sys

PARSER_C = 'C'
PARSER_PY = 'PYTHON'
try:
    import ovs._json
    PARSER = PARSER_C
except ImportError:
    PARSER = PARSER_PY

__pychecker__ = 'no-stringiter'

SPACES_PER_LEVEL = 2
dumper = functools.partial(json.dumps, separators=(",", ":"))


def to_stream(obj, stream, pretty=False, sort_keys=True):
    stream.write(dumper(obj, indent=SPACES_PER_LEVEL if pretty else None,
                        sort_keys=sort_keys))


def to_file(obj, name, pretty=False, sort_keys=True):
    with open(name, "w") as stream:
        to_stream(obj, stream, pretty, sort_keys)


def to_string(obj, pretty=False, sort_keys=True):
    return dumper(obj, indent=SPACES_PER_LEVEL if pretty else None,
                  sort_keys=sort_keys)


def from_stream(stream):
    p = Parser(check_trailer=True)
    while True:
        buf = stream.read(4096)
        if buf == "" or p.feed(buf) != len(buf):
            break
    return p.finish()


def from_file(name):
    stream = open(name, "r")
    try:
        return from_stream(stream)
    finally:
        stream.close()


def from_string(s):
    if not isinstance(s, str):
        # We assume the input is a string.  We will only hit this case for a
        # str in Python 2 which is not unicode, so we need to go ahead and
        # decode it.
        try:
            s = str(s, 'utf-8')
        except UnicodeDecodeError as e:
            seq = ' '.join(["0x%2x" % ord(c)
                           for c in e.object[e.start:e.end] if ord(c) >= 0x80])
            return "not a valid UTF-8 string: invalid UTF-8 sequence %s" % seq
    p = Parser(check_trailer=True)
    p.feed(s)
    return p.finish()


class Parser(object):
    # Maximum height of parsing stack. #
    MAX_HEIGHT = 1000

    def __new__(cls, *args, **kwargs):
        if PARSER == PARSER_C:
            return ovs._json.Parser(*args, **kwargs)
        return super(Parser, cls).__new__(cls)

    def __init__(self, check_trailer=False):
        self.check_trailer = check_trailer

        # Lexical analysis.
        self.lex_state = Parser.__lex_start
        self.buffer = ""
        self.line_number = 0
        self.column_number = 0
        self.byte_number = 0

        # Parsing.
        self.parse_state = Parser.__parse_start
        self.stack = []
        self.member_name = None

        # Parse status.
        self.done = False
        self.error = None

    def __lex_start_space(self, c):
        pass

    def __lex_start_alpha(self, c):
        self.buffer = c
        self.lex_state = Parser.__lex_keyword

    def __lex_start_token(self, c):
        self.__parser_input(c)

    def __lex_start_number(self, c):
        self.buffer = c
        self.lex_state = Parser.__lex_number

    def __lex_start_string(self, _):
        self.lex_state = Parser.__lex_string

    def __lex_start_error(self, c):
        if ord(c) >= 32 and ord(c) < 128:
            self.__error("invalid character '%s'" % c)
        else:
            self.__error("invalid character U+%04x" % ord(c))

    __lex_start_actions = {}
    for c in " \t\n\r":
        __lex_start_actions[c] = __lex_start_space
    for c in "abcdefghijklmnopqrstuvwxyz":
        __lex_start_actions[c] = __lex_start_alpha
    for c in "[{]}:,":
        __lex_start_actions[c] = __lex_start_token
    for c in "-0123456789":
        __lex_start_actions[c] = __lex_start_number
    __lex_start_actions['"'] = __lex_start_string

    def __lex_start(self, c):
        Parser.__lex_start_actions.get(
            c, Parser.__lex_start_error)(self, c)
        return True

    __lex_alpha = {}
    for c in "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ":
        __lex_alpha[c] = True

    def __lex_finish_keyword(self):
        if self.buffer == "false":
            self.__parser_input(False)
        elif self.buffer == "true":
            self.__parser_input(True)
        elif self.buffer == "null":
            self.__parser_input(None)
        else:
            self.__error("invalid keyword '%s'" % self.buffer)

    def __lex_keyword(self, c):
        if c in Parser.__lex_alpha:
            self.buffer += c
            return True
        else:
            self.__lex_finish_keyword()
            return False

    __number_re = re.compile("(-)?(0|[1-9][0-9]*)"
            r"(?:\.([0-9]+))?(?:[eE]([-+]?[0-9]+))?$")

    def __lex_finish_number(self):
        s = self.buffer
        m = Parser.__number_re.match(s)
        if m:
            sign, integer, fraction, exp = m.groups()
            if (exp is not None and
                (int(exp) > sys.maxsize or int(exp) < -sys.maxsize - 1)):
                self.__error("exponent outside valid range")
                return

            if fraction is not None and len(fraction.lstrip('0')) == 0:
                fraction = None

            sig_string = integer
            if fraction is not None:
                sig_string += fraction
            significand = int(sig_string)

            pow10 = 0
            if fraction is not None:
                pow10 -= len(fraction)
            if exp is not None:
                pow10 += int(exp)

            if significand == 0:
                self.__parser_input(0)
                return
            elif significand <= 2 ** 63:
                while pow10 > 0 and significand <= 2 ** 63:
                    significand *= 10
                    pow10 -= 1
                while pow10 < 0 and significand % 10 == 0:
                    significand //= 10
                    pow10 += 1
                if (pow10 == 0 and
                    ((not sign and significand < 2 ** 63) or
                     (sign and significand <= 2 ** 63))):
                    if sign:
                        self.__parser_input(-significand)
                    else:
                        self.__parser_input(significand)
                    return

            value = float(s)
            if value == float("inf") or value == float("-inf"):
                self.__error("number outside valid range")
                return
            if value == 0:
                # Suppress negative zero.
                value = 0
            self.__parser_input(value)
        elif re.match("-?0[0-9]", s):
            self.__error("leading zeros not allowed")
        elif re.match("-([^0-9]|$)", s):
            self.__error("'-' must be followed by digit")
        elif re.match(r"-?(0|[1-9][0-9]*)\.([^0-9]|$)", s):
            self.__error("decimal point must be followed by digit")
        elif re.search("e[-+]?([^0-9]|$)", s):
            self.__error("exponent must contain at least one digit")
        else:
            self.__error("syntax error in number")

    def __lex_number(self, c):
        if c in ".0123456789eE-+":
            self.buffer += c
            return True
        else:
            self.__lex_finish_number()
            return False

    __4hex_re = re.compile("[0-9a-fA-F]{4}")

    def __lex_4hex(self, s):
        if len(s) < 4:
            self.__error("quoted string ends within \\u escape")
        elif not Parser.__4hex_re.match(s):
            self.__error("malformed \\u escape")
        elif s == "0000":
            self.__error("null bytes not supported in quoted strings")
        else:
            return int(s, 16)

    @staticmethod
    def __is_leading_surrogate(c):
        """Returns true if 'c' is a Unicode code point for a leading
        surrogate."""
        return c >= 0xd800 and c <= 0xdbff

    @staticmethod
    def __is_trailing_surrogate(c):
        """Returns true if 'c' is a Unicode code point for a trailing
        surrogate."""
        return c >= 0xdc00 and c <= 0xdfff

    @staticmethod
    def __utf16_decode_surrogate_pair(leading, trailing):
        """Returns the unicode code point corresponding to leading surrogate
        'leading' and trailing surrogate 'trailing'.  The return value will not
        make any sense if 'leading' or 'trailing' are not in the correct ranges
        for leading or trailing surrogates."""
        #  Leading surrogate:         110110wwwwxxxxxx
        # Trailing surrogate:         110111xxxxxxxxxx
        #         Code point: 000uuuuuxxxxxxxxxxxxxxxx
        w = (leading >> 6) & 0xf
        u = w + 1
        x0 = leading & 0x3f
        x1 = trailing & 0x3ff
        return (u << 16) | (x0 << 10) | x1
    __unescape = {'"': u'"',
                  "\\": u"\\",
                  "/": u"/",
                  "b": u"\b",
                  "f": u"\f",
                  "n": u"\n",
                  "r": u"\r",
                  "t": u"\t"}

    def __lex_finish_string(self):
        inp = self.buffer
        out = u""
        while len(inp):
            backslash = inp.find('\\')
            if backslash == -1:
                out += inp
                break
            out += inp[:backslash]
            inp = inp[backslash + 1:]
            if inp == "":
                self.__error("quoted string may not end with backslash")
                return

            replacement = Parser.__unescape.get(inp[0])
            if replacement is not None:
                out += replacement
                inp = inp[1:]
                continue
            elif inp[0] != u'u':
                self.__error("bad escape \\%s" % inp[0])
                return

            c0 = self.__lex_4hex(inp[1:5])
            if c0 is None:
                return
            inp = inp[5:]

            if Parser.__is_leading_surrogate(c0):
                if inp[:2] != u'\\u':
                    self.__error("malformed escaped surrogate pair")
                    return
                c1 = self.__lex_4hex(inp[2:6])
                if c1 is None:
                    return
                if not Parser.__is_trailing_surrogate(c1):
                    self.__error("second half of escaped surrogate pair is "
                                 "not trailing surrogate")
                    return
                code_point = Parser.__utf16_decode_surrogate_pair(c0, c1)
                inp = inp[6:]
            else:
                code_point = c0
            out += chr(code_point)
        self.__parser_input('string', out)

    def __lex_string_escape(self, c):
        self.buffer += c
        self.lex_state = Parser.__lex_string
        return True

    def __lex_string(self, c):
        if c == '\\':
            self.buffer += c
            self.lex_state = Parser.__lex_string_escape
        elif c == '"':
            self.__lex_finish_string()
        elif ord(c) >= 0x20:
            self.buffer += c
        else:
            self.__error("U+%04X must be escaped in quoted string" % ord(c))
        return True

    def __lex_input(self, c):
        eat = self.lex_state(self, c)
        assert eat is True or eat is False
        return eat

    def __parse_start(self, token, unused_string):
        if token == '{':
            self.__push_object()
        elif token == '[':
            self.__push_array()
        else:
            self.__error("syntax error at beginning of input")

    def __parse_end(self, unused_token, unused_string):
        self.__error("trailing garbage at end of input")

    def __parse_object_init(self, token, string):
        if token == '}':
            self.__parser_pop()
        else:
            self.__parse_object_name(token, string)

    def __parse_object_name(self, token, string):
        if token == 'string':
            self.member_name = string
            self.parse_state = Parser.__parse_object_colon
        else:
            self.__error("syntax error parsing object expecting string")

    def __parse_object_colon(self, token, unused_string):
        if token == ":":
            self.parse_state = Parser.__parse_object_value
        else:
            self.__error("syntax error parsing object expecting ':'")

    def __parse_object_value(self, token, string):
        self.__parse_value(token, string, Parser.__parse_object_next)

    def __parse_object_next(self, token, unused_string):
        if token == ",":
            self.parse_state = Parser.__parse_object_name
        elif token == "}":
            self.__parser_pop()
        else:
            self.__error("syntax error expecting '}' or ','")

    def __parse_array_init(self, token, string):
        if token == ']':
            self.__parser_pop()
        else:
            self.__parse_array_value(token, string)

    def __parse_array_value(self, token, string):
        self.__parse_value(token, string, Parser.__parse_array_next)

    def __parse_array_next(self, token, unused_string):
        if token == ",":
            self.parse_state = Parser.__parse_array_value
        elif token == "]":
            self.__parser_pop()
        else:
            self.__error("syntax error expecting ']' or ','")

    def __parser_input(self, token, string=None):
        self.lex_state = Parser.__lex_start
        self.buffer = ""
        self.parse_state(self, token, string)

    def __put_value(self, value):
        top = self.stack[-1]
        if isinstance(top, dict):
            top[self.member_name] = value
        else:
            top.append(value)

    def __parser_push(self, new_json, next_state):
        if len(self.stack) < Parser.MAX_HEIGHT:
            if len(self.stack) > 0:
                self.__put_value(new_json)
            self.stack.append(new_json)
            self.parse_state = next_state
        else:
            self.__error("input exceeds maximum nesting depth %d" %
                         Parser.MAX_HEIGHT)

    def __push_object(self):
        self.__parser_push({}, Parser.__parse_object_init)

    def __push_array(self):
        self.__parser_push([], Parser.__parse_array_init)

    def __parser_pop(self):
        if len(self.stack) == 1:
            self.parse_state = Parser.__parse_end
            if not self.check_trailer:
                self.done = True
        else:
            self.stack.pop()
            top = self.stack[-1]
            if isinstance(top, list):
                self.parse_state = Parser.__parse_array_next
            else:
                self.parse_state = Parser.__parse_object_next

    def __parse_value(self, token, string, next_state):
        number_types = [int]
        number_types.extend([float])
        number_types = tuple(number_types)
        if token in [False, None, True] or isinstance(token, number_types):
            self.__put_value(token)
        elif token == 'string':
            self.__put_value(string)
        else:
            if token == '{':
                self.__push_object()
            elif token == '[':
                self.__push_array()
            else:
                self.__error("syntax error expecting value")
            return
        self.parse_state = next_state

    def __error(self, message):
        if self.error is None:
            self.error = ("line %d, column %d, byte %d: %s"
                          % (self.line_number, self.column_number,
                             self.byte_number, message))
            self.done = True

    def feed(self, s):
        i = 0
        while True:
            if self.done or i >= len(s):
                return i

            c = s[i]
            if self.__lex_input(c):
                self.byte_number += 1
                if c == '\n':
                    self.column_number = 0
                    self.line_number += 1
                else:
                    self.column_number += 1

                i += 1

    def is_done(self):
        return self.done

    def finish(self):
        if self.lex_state == Parser.__lex_start:
            pass
        elif self.lex_state in (Parser.__lex_string,
                                Parser.__lex_string_escape):
            self.__error("unexpected end of input in quoted string")
        else:
            self.__lex_input(" ")

        if self.parse_state == Parser.__parse_start:
            self.__error("empty input stream")
        elif self.parse_state != Parser.__parse_end:
            self.__error("unexpected end of input")

        if self.error is None:
            assert len(self.stack) == 1
            return self.stack.pop()
        else:
            return self.error



# Copyright (c) 2012 Nicira, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at:
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import argparse

import ovs.daemon
import ovs.unixctl
import ovs.unixctl.server

vlog = ovs.vlog.Vlog("test-unixctl")
exiting = False


def unixctl_exit(conn, unused_argv, aux):
    assert aux == "aux_exit"
    global exiting

    exiting = True
    conn.reply(None)


def unixctl_echo(conn, argv, aux):
    assert aux == "aux_echo"
    conn.reply(str(argv))


def unixctl_echo_error(conn, argv, aux):
    assert aux == "aux_echo_error"
    conn.reply_error(str(argv))


def unixctl_log(conn, argv, unused_aux):
    vlog.info(str(argv[0]))
    conn.reply(None)


def unixctl_block(conn, unused_argv, unused_aux):
    pass


def main():
    parser = argparse.ArgumentParser(
        description="Open vSwitch unixctl test program for Python")
    parser.add_argument("--unixctl", help="UNIXCTL socket location or 'none'.")

    ovs.daemon.add_args(parser)
    ovs.vlog.add_args(parser)
    args = parser.parse_args()
    ovs.daemon.handle_args(args)
    ovs.vlog.handle_args(args)

    ovs.daemon.daemonize_start()
    error, server = ovs.unixctl.server.UnixctlServer.create(args.unixctl)
    if error:
        ovs.util.ovs_fatal(error, "could not create unixctl server at %s"
                           % args.unixctl, vlog)

    ovs.unixctl.command_register("exit", "", 0, 0, unixctl_exit, "aux_exit")
    ovs.unixctl.command_register("echo", "[arg ...]", 1, 2, unixctl_echo,
                                 "aux_echo")
    ovs.unixctl.command_register("log", "[arg ...]", 1, 2, unixctl_log, None)
    ovs.unixctl.command_register("echo_error", "[arg ...]", 1, 2,
                                 unixctl_echo_error, "aux_echo_error")
    ovs.unixctl.command_register("block", "", 0, 0, unixctl_block, None)
    ovs.daemon.daemonize_complete()

    vlog.info("Entering run loop.")
    poller = ovs.poller.Poller()
    while not exiting:
        server.run()
        server.wait(poller)
        if exiting:
            poller.immediate_wake()
        poller.block()
    server.close()


if __name__ == '__main__':
    main()



#!/usr/bin/env python3
# Copyright (c) 2013, 2014, 2015, 2016, 2020 Nicira, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at:
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import optparse
import os
import shutil
import subprocess
import sys
import time

ENV = os.environ
HOME = ENV["HOME"]
PWD = os.getcwd()
OVS_SRC = HOME + "/ovs"
if os.path.exists(PWD + "/README.rst"):
    OVS_SRC = PWD  # Use current directory as OVS source tree
RUNDIR = OVS_SRC + "/_run"
BUILD_GCC = OVS_SRC + "/_build-gcc"
BUILD_CLANG = OVS_SRC + "/_build-clang"

options = None
parser = None
commands = []


def set_path(build):
    PATH = "%(ovs)s/utilities:%(ovs)s/ovsdb:%(ovs)s/vswitchd" % {"ovs": build}

    ENV["PATH"] = PATH + ":" + ENV["PATH"]


def _sh(*args, **kwargs):
    print("------> " + " ".join(args))
    shell = len(args) == 1
    if kwargs.get("capture", False):
        proc = subprocess.Popen(args, stdout=subprocess.PIPE, shell=shell)
        return proc.stdout.readlines()
    elif kwargs.get("check", True):
        subprocess.check_call(args, shell=shell)
    else:
        subprocess.call(args, shell=shell)


def uname():
    return _sh("uname", "-r", capture=True)[0].decode().strip()


def sudo():
    if os.geteuid() != 0:
        _sh(" ".join(["sudo"] + sys.argv), check=True)
        sys.exit(0)


def conf():
    tag()

    try:
        os.remove(OVS_SRC + "/Makefile")
    except OSError:
        pass

    configure = ["../configure",
                 "--prefix=" + RUNDIR, "--localstatedir=" + RUNDIR,
                 "--with-logdir=%s/log" % RUNDIR,
                 "--with-rundir=%s/run" % RUNDIR,
                 "--enable-silent-rules", "--with-dbdir=" + RUNDIR, "--silent"]

    cflags = "-g -fno-omit-frame-pointer"

    if options.werror:
        configure.append("--enable-Werror")

    if options.cache_time:
        configure.append("--enable-cache-time")

    if options.mandir:
        configure.append("--mandir=" + options.mandir)

    if options.with_dpdk:
        configure.append("--with-dpdk=" + options.with_dpdk)

    if options.optimize is None:
        options.optimize = 0

    cflags += " -O%s" % str(options.optimize)

    ENV["CFLAGS"] = cflags

    _sh("./boot.sh")

    try:
        os.mkdir(BUILD_GCC)
    except OSError:
        pass  # Directory exists.

    os.chdir(BUILD_GCC)
    _sh(*(configure + ["--with-linux=/lib/modules/%s/build" % uname()]))

    try:
        _sh("clang --version", check=True)
        clang = True
    except subprocess.CalledProcessError:
        clang = False

    try:
        _sh("sparse --version", check=True)
        sparse = True
    except subprocess.CalledProcessError:
        sparse = False

    if clang:
        try:
            os.mkdir(BUILD_CLANG)
        except OSError:
            pass  # Directory exists.

        ENV["CC"] = "clang"
        os.chdir(BUILD_CLANG)
        _sh(*configure)

    if sparse:
        c1 = "C=1"
    else:
        c1 = ""

    os.chdir(OVS_SRC)

    make_str = "\t$(MAKE) -C %s $@\n"

    mf = open(OVS_SRC + "/Makefile", "w")
    mf.write("all:\n%:\n")
    if clang:
        mf.write(make_str % BUILD_CLANG)
    mf.write("\t$(MAKE) -C %s %s $@\n" % (BUILD_GCC, c1))
    mf.write("\ncheck-valgrind:\n")
    mf.write("\ncheck:\n")
    mf.write(make_str % BUILD_GCC)
    mf.close()


commands.append(conf)


def make(args=""):
    make = "make -s -j 8 " + args
    _sh(make)


commands.append(make)


def check():
    flags = ""
    if options.jobs:
        flags += "-j%d " % options.jobs
    else:
        flags += "-j8 "
    if options.tests:
        for arg in str.split(options.tests):
            if arg[0].isdigit():
                flags += "%s " % arg
            else:
                flags += "-k %s " % arg
    ENV["TESTSUITEFLAGS"] = flags
    make("check")


commands.append(check)


def tag():
    ctags = ['ctags', '-R', '-f', '.tags']

    try:
        _sh(*(ctags + ['--exclude="datapath/"']))
    except:
        try:
            _sh(*ctags)  # Some versions of ctags don't have --exclude
        except:
            pass

    try:
        _sh('cscope', '-R', '-b')
    except:
        pass


commands.append(tag)


def kill():
    sudo()
    for proc in ["ovs-vswitchd", "ovsdb-server"]:
        if os.path.exists("%s/run/openvswitch/%s.pid" % (RUNDIR, proc)):
            _sh("ovs-appctl", "-t", proc, "exit", check=False)
            time.sleep(.1)
        _sh("killall", "-q", "-2", proc, check=False)


commands.append(kill)


def reset():
    sudo()
    kill()
    if os.path.exists(RUNDIR):
        shutil.rmtree(RUNDIR)
    for dp in _sh("ovs-dpctl dump-dps", capture=True):
        _sh("ovs-dpctl", "del-dp", dp.decode().strip())


commands.append(reset)


def run():
    sudo()
    kill()
    for d in ["log", "run"]:
        d = "%s/%s" % (RUNDIR, d)
        shutil.rmtree(d, ignore_errors=True)
        os.makedirs(d)

    pki_dir = RUNDIR + "/pki"
    if not os.path.exists(pki_dir):
        os.mkdir(pki_dir)
        os.chdir(pki_dir)
        _sh("ovs-pki init")
        _sh("ovs-pki req+sign ovsclient")
        os.chdir(OVS_SRC)

    if not os.path.exists(RUNDIR + "/conf.db"):
        _sh("ovsdb-tool", "create", RUNDIR + "/conf.db",
            OVS_SRC + "/vswitchd/vswitch.ovsschema")

    opts = ["--pidfile", "--log-file"]

    if (options.user == "") or (options.user == "root:root"):
        _sh("chown", "root:root", "-R", RUNDIR)
        if '--user' in sys.argv:
            sys.argv.remove("--user")
    else:
        _sh("chown", options.user, "-R", RUNDIR)
        opts = ["--user", options.user] + opts

    if (options.monitor):
        opts = ["--monitor"] + opts

    _sh(*(["ovsdb-server",
           "--remote=punix:%s/run/db.sock" % RUNDIR,
           "--remote=db:Open_vSwitch,Open_vSwitch,manager_options",
           "--private-key=db:Open_vSwitch,SSL,private_key",
           "--certificate=db:Open_vSwitch,SSL,certificate",
           "--bootstrap-ca-cert=db:Open_vSwitch,SSL,ca_cert",
           "--detach", "-vconsole:off"] + opts))

    _sh("ovs-vsctl --no-wait --bootstrap set-ssl %s/ovsclient-privkey.pem"
        " %s/ovsclient-cert.pem %s/vswitchd.cacert"
        % (pki_dir, pki_dir, pki_dir))
    version = _sh("ovs-vsctl --no-wait --version", capture=True)
    version = version[0].decode().strip().split()[3]
    root_uuid = _sh("ovs-vsctl --no-wait --bare list Open_vSwitch",
                    capture=True)[0].decode().strip()
    _sh("ovs-vsctl --no-wait set Open_vSwitch %s ovs_version=%s"
        % (root_uuid, version))

    build = BUILD_CLANG if options.clang else BUILD_GCC
    cmd = [build + "/vswitchd/ovs-vswitchd"]

    if options.dpdk:
        _sh("ovs-vsctl --no-wait set Open_vSwitch %s "
            "other_config:dpdk-init=true" % root_uuid)
        _sh("ovs-vsctl --no-wait set Open_vSwitch %s other_config:"
            "dpdk-extra=\"%s\"" % (root_uuid, ' '.join(options.dpdk)))
    else:
        _sh("ovs-vsctl --no-wait set Open_vSwitch %s "
            "other_config:dpdk-init=false" % root_uuid)

    if options.gdb:
        cmd = ["gdb", "--args"] + cmd
    elif options.valgrind:
        cmd = ["valgrind", "--track-origins=yes", "--leak-check=full",
               "--suppressions=%s/tests/glibc.supp" % OVS_SRC,
               "--suppressions=%s/tests/openssl.supp" % OVS_SRC] + cmd
    else:
        opts = opts + ["-vconsole:off", "--detach", "--enable-dummy"]
    _sh(*(cmd + opts))


commands.append(run)


def modinst():
    if not os.path.exists("/lib/modules"):
        print("Missing modules directory.  Is this a Linux system?")
        sys.exit(1)

    sudo()
    try:
        _sh("rmmod", "openvswitch")
    except subprocess.CalledProcessError:
        pass  # Module isn't loaded

    try:
        _sh("rm -f /lib/modules/%s/extra/openvswitch.ko" % uname())
        _sh("rm -f /lib/modules/%s/extra/vport-*.ko" % uname())
    except subprocess.CalledProcessError:
        pass  # Module isn't installed

    conf()
    make()
    make("modules_install")

    _sh("modprobe", "openvswitch")
    _sh("dmesg | grep openvswitch | tail -1")
    _sh("find /lib/modules/%s/ -iname vport-*.ko -exec insmod '{}' \\;"
        % uname())


commands.append(modinst)


def env():
    print("export PATH=" + ENV["PATH"])


commands.append(env)


def doc():
    parser.print_help()
    print("""
This program is designed to help developers build and run Open vSwitch without
necessarily needing to know the gory details. Given some basic requirements
(described below), it can be used to build and run Open vSwitch, keeping
runtime files in the user's home directory.

Basic Configuration:
    # This section can be run as a script on ubuntu systems.

    # First install the basic requirements needed to build Open vSwitch.
    sudo apt-get install git build-essential libtool autoconf pkg-config \\
            libssl-dev gdb libcap-ng-dev linux-headers-`uname -r`

    # Next clone the Open vSwitch source.
    git clone https://github.com/openvswitch/ovs.git %(ovs)s

    # Setup environment variables.
    `%(v)s env`

    # Build the switch.
    %(v)s conf make

    # Install the kernel module
    sudo insmod %(ovs)s/datapath/linux/openvswitch.ko

    # If needed, manually load all required vport modules:
    sudo insmod %(ovs)s/datapath/linux/vport-vxlan.ko
    sudo insmod %(ovs)s/datapath/linux/vport-geneve.ko
    [...]

    # Run the switch.
    %(v)s run

Commands:
    conf    - Configure the ovs source.
    make    - Build the source (must have been configured).
    check   - Run the unit tests.
    tag     - Run ctags and cscope over the source.
    kill    - Kill all running instances of ovs.
    reset   - Reset any runtime configuration in %(run)s.
    run     - Run ovs.
    modinst - Build ovs and install the kernel module.
    env     - Print the required path environment variable.
    doc     - Print this message.

Note:
    If running as non-root user, "kill", "reset", "run" and "modinst"
    will always run as the root user, by rerun the commands with "sudo".
""" % {"ovs": OVS_SRC, "v": sys.argv[0], "run": RUNDIR})
    sys.exit(0)


commands.append(doc)


def parse_subargs(option, opt_str, value, parser):
    subopts = []

    while parser.rargs:
        dpdkarg = parser.rargs.pop(0)
        if dpdkarg == "--":
            break
        subopts.append(dpdkarg)

    setattr(parser.values, option.dest, subopts)


def main():
    global options
    global parser

    description = "Open vSwitch developer configuration. Try `%prog doc`."
    cmd_names = [c.__name__ for c in commands]
    usage = "usage: %prog" + " [options] [%s] ..." % "|".join(cmd_names)
    parser = optparse.OptionParser(usage=usage, description=description)

    group = optparse.OptionGroup(parser, "conf")
    group.add_option("--disable-Werror", dest="werror", action="store_false",
                     default=True, help="compile without the Werror flag")
    group.add_option("--cache-time", dest="cache_time",
                     action="store_true", help="configure with cached timing")
    group.add_option("--mandir", dest="mandir", metavar="MANDIR",
                     help="configure the man documentation install directory")
    group.add_option("--with-dpdk", dest="with_dpdk", metavar="DPDK_BUILD",
                     help="built with dpdk libraries located at DPDK_BUILD")
    parser.add_option_group(group)

    group = optparse.OptionGroup(parser, "Optimization Flags")
    for i in ["s", "g"] + list(range(4)) + ["fast"]:
        group.add_option("--O%s" % str(i), dest="optimize",
                         action="store_const", const=i,
                         help="compile with -O%s" % str(i))
    parser.add_option_group(group)

    group = optparse.OptionGroup(parser, "check")
    group.add_option("-j", "--jobs", dest="jobs", metavar="N", type="int",
                     help="Run N tests in parallel")
    group.add_option("--tests", dest="tests", metavar="FILTER",
                     help="""run specific tests and/or a test category
                          eg, --tests=\"1-10 megaflow\"""")
    parser.add_option_group(group)

    group = optparse.OptionGroup(parser, "run")
    group.add_option("-g", "--gdb", dest="gdb", action="store_true",
                     help="run ovs-vswitchd under gdb")
    group.add_option("--valgrind", dest="valgrind", action="store_true",
                     help="run ovs-vswitchd under valgrind")
    group.add_option("--dpdk", dest="dpdk", action="callback",
                     callback=parse_subargs,
                     help="run ovs-vswitchd with dpdk subopts (ended by --)")
    group.add_option("--clang", dest="clang", action="store_true",
                     help="Use binaries built by clang")
    group.add_option("--user", dest="user", action="store", default="",
                     help="run all daemons as a non root user")
    group.add_option("--monitor", dest="monitor", action="store_true",
                     help="run daemons with --monitor option")

    parser.add_option_group(group)

    options, args = parser.parse_args()

    for arg in args:
        if arg not in cmd_names:
            print("Unknown argument " + arg)
            doc()

    if options.clang:
        set_path(BUILD_CLANG)
    else:
        set_path(BUILD_GCC)

    try:
        os.chdir(OVS_SRC)
    except OSError:
        print("Missing %s." % OVS_SRC)
        doc()

    for arg in args:
        for cmd in commands:
            if arg == cmd.__name__:
                cmd()


if __name__ == '__main__':
    main()



# Copyright (c) 2010, 2011 Nicira, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at:
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import atexit
import os
import signal
import sys

import ovs.vlog

_hooks = []
vlog = ovs.vlog.Vlog("fatal-signal")


def add_hook(hook, cancel, run_at_exit):
    _init()
    _hooks.append((hook, cancel, run_at_exit))


def fork():
    """Clears all of the fatal signal hooks without executing them.  If any of
    the hooks passed a 'cancel' function to add_hook(), then those functions
    will be called, allowing them to free resources, etc.

    Following a fork, one of the resulting processes can call this function to
    allow it to terminate without calling the hooks registered before calling
    this function.  New hooks registered after calling this function will take
    effect normally."""
    global _hooks
    for hook, cancel, run_at_exit in _hooks:
        if cancel:
            cancel()

    _hooks = []


_added_hook = False
_files = {}


def add_file_to_unlink(file):
    """Registers 'file' to be unlinked when the program terminates via
    sys.exit() or a fatal signal."""
    global _added_hook
    if not _added_hook:
        _added_hook = True
        add_hook(_unlink_files, _cancel_files, True)
    _files[file] = None


def add_file_to_close_and_unlink(file, fd=None):
    """Registers 'file' to be unlinked when the program terminates via
    sys.exit() or a fatal signal and the 'fd' to be closed. On Windows a file
    cannot be removed while it is open for writing."""
    global _added_hook
    if not _added_hook:
        _added_hook = True
        add_hook(_unlink_files, _cancel_files, True)
    _files[file] = fd


def remove_file_to_unlink(file):
    """Unregisters 'file' from being unlinked when the program terminates via
    sys.exit() or a fatal signal."""
    if file in _files:
        del _files[file]


def unlink_file_now(file):
    """Like fatal_signal_remove_file_to_unlink(), but also unlinks 'file'.
    Returns 0 if successful, otherwise a positive errno value."""
    error = _unlink(file)
    if error:
        vlog.warn("could not unlink \"%s\" (%s)" % (file, os.strerror(error)))
    remove_file_to_unlink(file)
    return error


def _unlink_files():
    for file_ in _files:
        if sys.platform == "win32" and _files[file_]:
            _files[file_].close()
        _unlink(file_)


def _cancel_files():
    global _added_hook
    global _files
    _added_hook = False
    _files = {}


def _unlink(file_):
    try:
        os.unlink(file_)
        return 0
    except OSError as e:
        return e.errno


def _signal_handler(signr, _):
    _call_hooks(signr)

    # Re-raise the signal with the default handling so that the program
    # termination status reflects that we were killed by this signal.
    signal.signal(signr, signal.SIG_DFL)
    os.kill(os.getpid(), signr)


def _atexit_handler():
    _call_hooks(0)


recurse = False


def _call_hooks(signr):
    global recurse
    if recurse:
        return
    recurse = True

    for hook, cancel, run_at_exit in _hooks:
        if signr != 0 or run_at_exit:
            hook()


_inited = False


def _init():
    global _inited
    if not _inited:
        _inited = True
        if sys.platform == "win32":
            signals = [signal.SIGTERM, signal.SIGINT]
        else:
            signals = [signal.SIGTERM, signal.SIGINT, signal.SIGHUP,
                       signal.SIGALRM]

        for signr in signals:
            if signal.getsignal(signr) == signal.SIG_DFL:
                signal.signal(signr, _signal_handler)
        atexit.register(_atexit_handler)


def signal_alarm(timeout):
    if not timeout:
        env_timeout = os.environ.get('OVS_CTL_TIMEOUT')
        if env_timeout:
            timeout = int(env_timeout)
    if not timeout:
        return

    if sys.platform == "win32":
        import time
        import threading

        class Alarm (threading.Thread):
            def __init__(self, timeout):
                super(Alarm, self).__init__()
                self.timeout = timeout
                self.setDaemon(True)

            def run(self):
                time.sleep(self.timeout)
                os._exit(1)

        alarm = Alarm(timeout)
        alarm.start()
    else:
        signal.alarm(timeout)
